require "math"

function newPaddedImage(source)

    -- rounds up to a power-of-two image
    -- based obviously on the code example on the love wiki, but modified
    -- to take an existing imagedata as arg, rather than a filename
    
    local w = source:getWidth()
    local h = source:getHeight()
    
    -- Find closest power-of-two.
    local wp = math.pow(2, math.ceil(math.log(w)/math.log(2)))
    local hp = math.pow(2, math.ceil(math.log(h)/math.log(2)))
    
    -- Only pad if needed:
    if wp ~= w or hp ~= h then
        local padded = love.image.newImageData(wp, hp)
        padded:paste(source, 0, 0)
        return love.graphics.newImage(padded)
    end
    
    return love.graphics.newImage(source)
end

function love.load()
   local random = math.random
   plasma = {}   -- table which will hold our stars
   local p = plasma -- for speed
   width = love.graphics.getWidth()
   height = love.graphics.getHeight()
   local i
   for i=0,width-1 do
       p[i]={}
   end
   make_plasma(width, height, 128 )
   
   main_imagedata = love.image.newImageData(width,height)
   for i=0,width-1 do
       for j=0,height-1 do
           local q = p[i][j]
	   local r = q/2
	   local g = q
	   local b = q/2+128
           main_imagedata:setPixel(i,j,r,g,b,255)
       end
   end
   
  main_image = newPaddedImage(main_imagedata)
end

function make_plasma(width,height,noisiness)
   local random = math.random
   local w = width
   local h = height
   
   plasma[0][0]=random(0,255)
   plasma[w-1][0]=random(0,255)
   plasma[0][h-1]=random(0,255)
   plasma[w-1][h-1]=random(0,255)
   
   plasma_recurse(0,0,w-1,h-1,noisiness,0)
end

function plasma_recurse(x0,y0,x1,y1,noisiness,depth)

    -- end recursion if we cannot subdivide any more
    if x1<=x0+1 and y1<=y0+1 then return end
    
    assert( depth<20 )
    local p = plasma
    local floor = math.floor
    local random = math.random
    local max = math.max
    local min = math.min
    local xmid=floor((x0+x1)/2)
    local ymid=floor((y0+y1)/2)
    
    if p[xmid][y0]==nil then p[xmid][y0]=min(255,max(0,floor((p[x0][y0]+p[x1][y0])/2+random(0,noisiness)-random(0,noisiness)))) end
    if p[xmid][y1]==nil then p[xmid][y1]=min(255,max(0,floor((p[x0][y1]+p[x1][y1])/2+random(0,noisiness)-random(0,noisiness)))) end
    if p[x0][ymid]==nil then p[x0][ymid]=min(255,max(0,floor((p[x0][y0]+p[x0][y1])/2+random(0,noisiness)-random(0,noisiness)))) end
    if p[x1][ymid]==nil then p[x1][ymid]=min(255,max(0,floor((p[x1][y0]+p[x1][y1])/2+random(0,noisiness)-random(0,noisiness)))) end
    if p[xmid][ymid]==nil then p[xmid][ymid]=min(255,max(0,floor((p[x0][y0]+p[x0][y1]+p[x1][y0]+p[x1][y1])/4+random(0,noisiness)-random(0,noisiness)))) end
    
    local nnoise = floor(noisiness/2)
    plasma_recurse(x0,y0,xmid,ymid,nnoise,depth+1)
    plasma_recurse(xmid,y0,x1,ymid,nnoise,depth+1)
    plasma_recurse(x0,ymid,xmid,y1,nnoise,depth+1)
    plasma_recurse(xmid,ymid,x1,y1,nnoise,depth+1)
end

function love.draw()
    love.graphics.draw(main_image,0,0)
end

